/*
 * @file <src/CoreLoopFunctions.cpp>
 *
 * @author Antoine Ligot - <aligot@ulb.ac.be>
 * @author Ken Hasselmann - <khasselm@ulb.ac.be>
 *
 * @package experiments-loop-functions
 *
 * @license MIT License
 */


#include "CoreLoopFunctions.h"

/****************************************/
/****************************************/

void CoreLoopFunctions::Init(argos::TConfigurationNode& t_tree) {
  m_pcRng = CRandom::CreateRNG("argos");
  LOG<< "RNG:" << m_pcRng << std::endl;
  TConfigurationNode cParametersNode;
  try {
    cParametersNode = GetNode(t_tree, "params");
    GetNodeAttributeOrDefault(cParametersNode, "number_robots", m_unNumberRobots, (UInt32) 1);
    GetNodeAttributeOrDefault(cParametersNode, "dist_radius", m_fDistributionRadius, (Real) 0);
  } catch(std::exception e) {
    LOGERR << e.what() << std::endl;
  }
  MoveRobots();
  //PositionRobots();
}

/****************************************/
/****************************************/

void CoreLoopFunctions::Reset() {
  MoveRobots();
}

/****************************************/
/****************************************/

CoreLoopFunctions::~CoreLoopFunctions() {}

/****************************************/
/****************************************/
/************/
//NB :
/************/
//This function is legacy, it should not be used because it does not allow
// the use of the tracking system of iridia.
// To use the tracking system, the robots objects should the created in the
// xml file, and then just moved by the MoveRobots() function.
/************/

// void CoreLoopFunctions::PositionRobots() {
//   CEPuckEntity* pcEpuck;
//   bool bPlaced = false;
//   UInt32 unTrials;
//   for(UInt32 i = 1; i < m_unNumberRobots + 1; ++i) {
//     std::ostringstream id;
//     id << "epuck" << i;
//     pcEpuck = new CEPuckEntity(id.str().c_str(),
//                                "automode",
//                                CVector3(0,0,0),
//                                CQuaternion().FromEulerAngles(CRadians::ZERO,CRadians::ZERO,CRadians::ZERO));
//     AddEntity(*pcEpuck);
//     // Choose position at random
//     unTrials = 0;
//     do {
//        ++unTrials;
//        CVector3 cEpuckPosition = GetRandomPosition();
//        bPlaced = MoveEntity((*pcEpuck).GetEmbodiedEntity(),
//                             cEpuckPosition,
//                             CQuaternion().FromEulerAngles(m_pcRng->Uniform(CRange<CRadians>(CRadians::ZERO,CRadians::TWO_PI)),
//                             CRadians::ZERO,CRadians::ZERO),false);
//     } while(!bPlaced && unTrials < 100);
//     if(!bPlaced) {
//        THROW_ARGOSEXCEPTION("Can't place robot");
//     }
//   }
// }

/****************************************/
/****************************************/

void CoreLoopFunctions::MoveRobots() {
  CEPuckEntity* pcEpuck;
  bool bPlaced = false;
  UInt32 unTrials;
  CSpace::TMapPerType& tEpuckMap = GetSpace().GetEntitiesByType("epuck");
  for (CSpace::TMapPerType::iterator it = tEpuckMap.begin(); it != tEpuckMap.end(); ++it) {
    pcEpuck = any_cast<CEPuckEntity*>(it->second);
    // Choose position at random
    unTrials = 0;
    do {
       ++unTrials;
       CVector3 cEpuckPosition = GetRandomPosition();
       bPlaced = MoveEntity(pcEpuck->GetEmbodiedEntity(),
                            cEpuckPosition,
                            CQuaternion().FromEulerAngles(m_pcRng->Uniform(CRange<CRadians>(CRadians::ZERO,CRadians::TWO_PI)),
                            CRadians::ZERO,CRadians::ZERO),false);
    } while(!bPlaced && unTrials < 100);
    if(!bPlaced) {
       THROW_ARGOSEXCEPTION("Can't place robot");
    }
  }
}

/****************************************/
/****************************************/

void CoreLoopFunctions::RemoveRobots() {
  for(UInt32 i = 1; i < m_unNumberRobots + 1; ++i) {
    std::ostringstream id;
    id << "epuck" << i;
    RemoveEntity(id.str().c_str());
  }
}
